using System;
using System.Reflection;
using System.Collections;
using Server.Network;
using Server.Prompts;
using Server.Multis;
using Server.Multis.Deeds;
using Server.Items;

namespace Server.Gumps
{
	public class HouseGump : Gump
	{
		private BaseHouse m_House;

		public HouseGump( Mobile from, BaseHouse house ) : base( 0, 0 )
		{
			m_House = house;

			bool isOwner = from.Serial == m_House.Owner.Serial ? true : false;
			bool isCoOwn;
			bool isFriend;

			if ( from.AccessLevel >= AccessLevel.GameMaster )
				isOwner = true;

			if ( isOwner )
			{
				isCoOwn = isFriend = true;
			}
			else
			{
				isCoOwn = m_House.IsCoOwner( from );
				if ( isCoOwn )
					isFriend = true;
				else
					isFriend = m_House.IsFriend( from );
			}

			if ( !isFriend )
			{
				//m_House.Sign.OnSingleClick( from );
				AddPage( 0 );

				AddImageTiled( 0, 0, 143, 101, 0x64 );

				AddPage( 1 );

				AddLabelCropped( 23, 21, 99, 60, 0x00, m_House.Sign.Name );

				return;
			}

			AddPage( 0 );

			AddBackground( 0, 0, 450, 350, 0x13BE ); 
			AddImageTiled( 5, 5, 440, 340, 0xA2C );
			//AddAlphaRegion( 5, 5, 440, 340 );

			AddImageTiled( 150, 0, 143, 101, 0x64 );
			AddLabelCropped( 173, 21, 99, 60, 0x00, m_House.Sign.Name );

			AddButton( 60, 110, 5606, 5606, 1, GumpButtonType.Page, 1 );
			AddLabel( 80, 108, 1152, "Info" );

			AddButton( 180, 110, 5606, 5606, 2, GumpButtonType.Page, 2 );
			AddLabel( 200, 108, 1152, "Friends" );

			AddButton( 320, 110, 5606, 5606, 3, GumpButtonType.Page, 3 );
			AddLabel( 340, 108, 1152, "Options" );

			AddPage( 1 ); // info

			AddLabel( 40, 140, 482, "Owned By: " + m_House.Owner.Name );
			AddLabel( 40, 160, 482, "Secured Conts: " + m_House.Secures.Count );
			AddLabel( 190, 160, 482, "Out Of: " + m_House.MaxSecures );
			AddLabel( 40, 180, 482, "Lock Downs: " + m_House.LockDownCount );
			AddLabel( 190, 180, 482, "Out Of: " + m_House.MaxLockDowns );
			//AddLabel( 40, 220, 482, "Visits: 9587" );

			AddButton( 310, 315, 5601, 5601, 4, GumpButtonType.Reply, 0 );
			AddLabel( 330, 313, 482, "Rename House" );

			AddPage( 2 ); // friends

			AddButton( 40, 140, 5601, 5601, 5, GumpButtonType.Reply, 0 );
			AddLabel( 60, 138, 482, "List Co-Owners" );

			if ( isOwner )
			{
				AddButton( 40, 160, 5601, 5601, 6, GumpButtonType.Reply, 0 );
				AddLabel( 60, 158, 482, "Add Co-Owner" );

				AddButton( 40, 180, 5601, 5601, 7, GumpButtonType.Reply, 0 );
				AddLabel( 60, 178, 482, "Remove Co-Owner" );

				AddButton( 40, 200, 5601, 5601, 8, GumpButtonType.Reply, 0 );
				AddLabel( 60, 198, 482, "Clear Co-Owners" );
			}


			AddButton( 260, 140, 5601, 5601, 9, GumpButtonType.Reply, 0 );
			AddLabel( 280, 138, 482, "List Friends" );

			if ( isCoOwn )
			{
				AddButton( 260, 160, 5601, 5601, 10, GumpButtonType.Reply, 0 );
				AddLabel( 280, 158, 482, "Add Friend" );

				AddButton( 260, 180, 5601, 5601, 11, GumpButtonType.Reply, 0 );
				AddLabel( 280, 178, 482, "Remove Friend" );

				if ( isOwner )
				{
					AddButton( 260, 200, 5601, 5601, 12, GumpButtonType.Reply, 0 );
					AddLabel( 280, 198, 482, "Clear Friends" );			
				}
			}

			AddButton( 115, 240, 5601, 5601, 13, GumpButtonType.Reply, 0 );
			AddLabel( 135, 238, 482, "Eject" );

			AddButton( 115, 260, 5601, 5601, 14, GumpButtonType.Reply, 0 );
			AddLabel( 135, 258, 482, "List Bans" );

			if ( isCoOwn )
			{
				AddButton( 115, 280, 5601, 5601, 15, GumpButtonType.Reply, 0 );
				AddLabel( 135, 278, 482, "Add Ban" );

				AddButton( 115, 300, 5601, 5601, 16, GumpButtonType.Reply, 0 );
				AddLabel( 138, 298, 482, "Remove Ban" );
			}

			if ( isOwner )
			{
				AddPage( 3 ); // options

				AddButton( 40, 140, 5601, 5601, 17, GumpButtonType.Reply, 0 );
				AddLabel( 60, 138, 482, "Transfer ownership of the house" );

				AddButton( 40, 160, 5601, 5601, 18, GumpButtonType.Reply, 0 );
				AddLabel( 60, 158, 482, "Change the sign type" );

				AddButton( 40, 180, 5601, 5601, 19, GumpButtonType.Reply, 0 );
				AddLabel( 60, 178, 482, "Change the house locks" );

				AddButton( 40, 200, 5601, 5601, 20, GumpButtonType.Reply, 0 );
				AddLabel( 60, 198, 482, "Demolish house & get a deed back" );
			}
		}

		public override void OnResponse(NetState sender, RelayInfo info)
		{
			bool isOwner = m_House.Owner.Serial == sender.Mobile.Serial;
			if ( sender.Mobile.AccessLevel >= AccessLevel.GameMaster )
				isOwner = true;

			switch ( info.ButtonID )
			{
				//1-3 are pages
				case 4://rename sign
					sender.Mobile.Prompt = new RenamePrompt( m_House );
					sender.Mobile.SendLocalizedMessage( 501302 );//What should the sign say
					break;
				case 5://list coowners
					break;
				case 6://add coowner
					sender.Mobile.Target = new CoOwnerTarget( true, m_House );
					sender.Mobile.SendLocalizedMessage( 501328 );//Who do you wish to make coowner
					break;
				case 7://remove coowner
					sender.Mobile.Target = new CoOwnerTarget( false, m_House );
					sender.Mobile.SendMessage( "Target the person to remove from the Co-Owners list." );
					break;
				case 8://clear co-owners
					if ( isOwner )
					{
						m_House.CoOwners.Clear();
						sender.Mobile.SendLocalizedMessage( 501333 );//CoOwner list cleared
					}
					else
					{
						sender.Mobile.SendLocalizedMessage( 501329 );//Only the owner may remove coowners
					}	
					break;
				case 9://list friends
					break;

				case 10://add friend
					sender.Mobile.Target = new HouseFriendTarget( true, m_House );
					sender.Mobile.SendLocalizedMessage( 501317 );//Target the person to friend
					break;
				case 11:
					sender.Mobile.Target = new HouseFriendTarget( false, m_House );
					sender.Mobile.SendMessage( "Target the person to remove from the friends list." );
					break;
				case 12://clear friends
					if ( isOwner )
					{
						m_House.Friends.Clear();
						sender.Mobile.SendLocalizedMessage( 501332 );//Friends list cleared
					}
					else
					{
						sender.Mobile.SendLocalizedMessage( 501319 );//Only the owner may remove friends
					}
					break;

				case 13://kick
					sender.Mobile.Target = new HouseKickTarget( m_House );
					sender.Mobile.SendLocalizedMessage( 501326 );//target the person to eject
					break;

				case 14://list bans
					break;

				case 15://add ban
					sender.Mobile.Target = new HouseBanTarget( true, m_House );
					sender.Mobile.SendLocalizedMessage( 501325 );//target the person to ban
					break;

				case 16://remove ban
					sender.Mobile.Target = new HouseBanTarget( false, m_House );
					sender.Mobile.SendMessage( "Select the person to un-ban." );
					break;

				case 17://transfer ownership
					if ( isOwner )
					{
						sender.Mobile.Target = new HouseOwnerTarget( m_House );
						sender.Mobile.SendLocalizedMessage( 501309 );//target the person whom you wish to give the house
					}
					else
					{
						sender.Mobile.SendLocalizedMessage( 501310 );//Only the owner may do this
					}
					break;
				case 18://change sign
					break;

				case 19://change locks
					if ( isOwner )
					{
						Key k1 = new Key( KeyType.Gold ), k2 = new Key( KeyType.Gold );
						k1.KeyValue = k2.KeyValue = Key.RandomValue();
						k1.Newbied = k2.Newbied = true;
						sender.Mobile.BankBox.TryDropItem( sender.Mobile, k2, false );
						Container pack = sender.Mobile.Backpack;
						if ( pack == null || !pack.TryDropItem( sender.Mobile, k1, true ) )
						{
							k1.MoveToWorld( sender.Mobile.Location, sender.Mobile.Map );
						}

						foreach ( BaseDoor d in m_House.Doors )
						{
							if ( d.KeyValue != 0 )
								d.KeyValue = k1.KeyValue;
						}
					}

					break;
				case 20://re-deed
					if ( isOwner )
					{
						HouseDeed deed = m_House.GetDeed();
						if ( deed == null )
						{
							sender.Mobile.SendMessage( "Could not create a deed for this house." );
							return;
						}

						sender.Mobile.BankBox.DropItem( deed );
						m_House.Delete();
						sender.Mobile.SendLocalizedMessage( 1011281 );//Are you sure you want to turn your house into a deed?
						sender.Mobile.SendLocalizedMessage( 1011280 );//Yes, demolish the house and place a deed in my bankbox.
					}
					break;
			}
		}
	}
}

namespace Server.Prompts
{
	public class RenamePrompt : Prompt
	{
		private BaseHouse m_House;

		public RenamePrompt( BaseHouse house )
		{
			m_House = house;
		}

		public override void OnResponse( Mobile from, string text )
		{
			if ( m_House.IsFriend( from ) )
			{
				m_House.Sign.Name = text;
				from.SendMessage( "Sign changed." );
			}
		}
	}
}

