using System;
using System.Collections;
using Server.Targeting;
using Server.Network;


namespace Server.Mobiles
{
	public enum AIType
	{
		AI_Use_Default,
		AI_Meelee,
		AI_Animal,
		AI_Archer,
		AI_Healer,
		AI_Berseck
	}

	public enum AcquireType
	{
		Strongest,
		Weakest,
		Closest,
		MageStrongest,
	}

	public abstract class BaseAI
	{
		public Timer	m_Timer;

		public BaseCreature m_Mobile;

		public BaseAI(BaseCreature m)

		{
			m_Mobile = m;

			m_Timer = new AITimer( this );
			m_Timer.Start();
		}

		public abstract bool Think();

		/*
		 *  Walk stupidly at random but slowly
		 */ 
		public virtual void DoWalkRandomSlowly(int iChanceToNotMove, int iSteps)
		{
			if ( m_Mobile.Deleted )
				return;

			for (int i=0; i<iSteps; i++)
			{
				int iRndMove = Utility.Random(0,8 + ( 9*iChanceToNotMove ));

				switch (iRndMove)
				{
					case 0:
						m_Mobile.Move(Direction.Up);
						break;
					case 1:
						m_Mobile.Move(Direction.North);
						break;
					case 2:
						m_Mobile.Move(Direction.Left);
						break;
					case 3:
						m_Mobile.Move(Direction.West);
						break;
					case 5:
						m_Mobile.Move(Direction.Down);
						break;
					case 6:
						m_Mobile.Move(Direction.South);
						break;
					case 7:
						m_Mobile.Move(Direction.Right);
						break;
					case 8:
						m_Mobile.Move(Direction.East);
						break;
				}
			}
		}

		/*
		 *  Walk stupidly at random with chance to change to stay in same direction
		 */ 
		public virtual void DoWalkRandomNonStop(int iChanceToDir, int iSteps)
		{
			if ( m_Mobile.Deleted )
				return;

			for (int i=0; i<iSteps; i++)
			{
				int iRndMove = Utility.Random(0,8 + ( 9*iChanceToDir ));

				switch (iRndMove)
				{
					case 0:
						m_Mobile.Move(Direction.Up);
						break;
					case 1:
						m_Mobile.Move(Direction.North);
						break;
					case 2:
						m_Mobile.Move(Direction.Left);
						break;
					case 3:
						m_Mobile.Move(Direction.West);
						break;
					case 5:
						m_Mobile.Move(Direction.Down);
						break;
					case 6:
						m_Mobile.Move(Direction.South);
						break;
					case 7:
						m_Mobile.Move(Direction.Right);
						break;
					case 8:
						m_Mobile.Move(Direction.East);
						break;
					default:
						m_Mobile.Move(m_Mobile.Direction);
						break;
				}
			}
		}

		/*
		 *  Walk at range distance from mobile
		 * 
		 *	iSteps : Number of steps
		 *	bRun   : Do we run
		 *	iWantDistMin : The minimum distance we want to be
		 *  iWantDistMax : The maximum distance we want to be
		 * 
		 */ 
		public virtual bool DoWalkMobileRange(Mobile m, int iSteps, bool bRun, int iWantDistMin, int iWantDistMax)
		{
			if ( m_Mobile.Deleted )
				return false;

			if (m != null)
			{
				for (int i=0; i<iSteps; i++)
				{
					// Get the curent distance
					int iCurrDist = (int)m_Mobile.GetDistanceToSqrt( m );

					if (iCurrDist < iWantDistMin || iCurrDist > iWantDistMax)
					{
						Direction dirTo;

						if ( iCurrDist > iWantDistMax  )
						{
							// Let's get closer
							dirTo = m_Mobile.GetDirectionTo( m );
						}
						else
						{
							dirTo = m.GetDirectionTo( m_Mobile );
						}

						// Add the run flag
						if (bRun)
						{
							dirTo = dirTo | Direction.Running;
						}


						if ( !m_Mobile.Move( dirTo ) )
						{
							// Here will call a PATH FINDING system
							// For now, we go left or right
							if ( Utility.Random(0,10) >= 5 )
							{
								m_Mobile.DebugSay( "Block: Turn Left" );
								m_Mobile.Turn( 2 );

								if ( !m_Mobile.Move( m_Mobile.Direction ) )
								{
									m_Mobile.DebugSay( "Block the other side" );
									m_Mobile.Turn( 4 );
									m_Mobile.Move( m_Mobile.Direction );
								}
							}
							else
							{
								m_Mobile.DebugSay( "Block: Turn right" );
								m_Mobile.Turn( -2 );

								if ( !m_Mobile.Move( m_Mobile.Direction ) )
								{
									m_Mobile.DebugSay( "Block the other side" );
									m_Mobile.Turn( 4 );
									m_Mobile.Move( m_Mobile.Direction );
								}
							}

							
						}
					}
					else
					{
						return true;
					}
				}

				// Get the curent distance
				int iNewDist = (int)m_Mobile.GetDistanceToSqrt( m );

				if (iNewDist >= iWantDistMin && iNewDist <= iWantDistMax)
				{
					return true;                    
				}
				else
				{
					return false;
				}
			}

			return false;
		}

		/*
		 * Here we check to aquire a target from our surronding
		 * 
		 *  iRange : The range
		 *  acqType : A type of aquire we want (closest, stronguest, etc
		 *  bPlayerOnly : Don't bother with other creature or NPC, want a player
		 *  bFacFriend : Check peoples in my faction
		 *  bFacFoe : Check peoples in other factions
		 * 
		 */ 
		public virtual bool AquireFocusMob(int iRange, AcquireType acqType, bool bPlayerOnly, bool bFacFriend, bool bFacFoe)
		{
			if ( m_Mobile.Deleted )
				return false;

			Map map = m_Mobile.Map;

			if ( map != null )
			{
				Mobile newFocusMob = null;
				double val = double.MinValue;

				IPooledEnumerable eable = map.GetMobilesInRange( m_Mobile.Location, iRange);

				foreach ( Mobile m in eable )
				{
					bool bCheckIt = false;

					//
					// Basic check
					//
					if ( ( bPlayerOnly && m.Player ) ||  !bPlayerOnly )
					{
						if ( m.AccessLevel < AccessLevel.GameMaster )
						{
							if ( !m.Deleted )
							{
								if ( m != m_Mobile )
								{
									if (m_Mobile.CanSee( m ) )
									{
										bCheckIt = true;
									}
								}
							}
						}
					}

					//
					// Faction check
					//
					if (bCheckIt) // alrealy passed the others tests
					{
						if ( m is BaseCreature )
						{
							if (bFacFriend) // we want allies
							{
								// but its not a friend
								if ( ((BaseCreature)m).Faction != m_Mobile.Faction )
								{
                                    bCheckIt = false;		                            
								}
							}

							if (bFacFoe) // we want foes
							{
								// but its a friend
								if ( ((BaseCreature)m).Faction == m_Mobile.Faction )
								{
									bCheckIt = false;		                            
								}
							}
						}
						else // is a player
						{
							// If it's a player, it can't be in your faction
							if ( bFacFriend )
							{
								bCheckIt = false;
							}
						}
					}

					if (bCheckIt)
					{
						double theirVal = m_Mobile.GetValueFrom( m , acqType, bPlayerOnly);

						if ( theirVal > val )
						{
							newFocusMob = m;
							val = theirVal;
						}
					}
				}

				eable.Free();

				m_Mobile.FocusMob = newFocusMob;
			}

			if (m_Mobile.FocusMob != null)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/*
		 * 
		 *  A Detect Hiden prototype
		 * 
		 */ 
		public virtual void DetectHiden()
		{
			if ( m_Mobile.Deleted )
				return;

			if (m_Mobile.Combatant == null && m_Mobile.FocusMob == null)
			{
				m_Mobile.DebugSay("I check for hiden");
				
				Map map = m_Mobile.Map;

				if ( map != null )
				{
					double srcSkill = m_Mobile.Skills[SkillName.DetectHidden].Value;

					IPooledEnumerable inRange = map.GetMobilesInRange( m_Mobile.Location, m_Mobile.RangePerception);

					foreach ( Mobile trg in inRange )
					{
						if ( trg != m_Mobile && trg.Hidden && trg.AccessLevel < AccessLevel.GameMaster  )
						{
							m_Mobile.DebugSay("I try to detect " + trg.Name);

							double trgSkill = trg.Skills[SkillName.Hiding].Value;

							double x = (srcSkill * (Utility.Random( 25 ) + 75));
							double y = (trgSkill * (Utility.Random( 35 ) + 65));

							if ( x > y )
							{
								trg.Hidden = false;
								trg.SendLocalizedMessage( 500814 ); // You have been revealed!
							}
						}
					}

					inRange.Free();
				}
			}
			else
			{
				m_Mobile.DebugSay("I have no time to check hiden");
			}
		}

		/*
		 *  The mobile changed it speed, we must ajust the timer
		 */ 
		public virtual void OnCurrentSpeedChanged()
		{
			m_Timer.Stop();
			m_Timer.Delay = TimeSpan.FromSeconds( m_Mobile.CurrentSpeed );
			m_Timer.Interval = TimeSpan.FromSeconds( m_Mobile.CurrentSpeed );
			m_Timer.Start();
		}
		

		/*
		 *  The Timer object
		 */ 
		private class AITimer : Timer
		{
			private BaseAI m_Owner;
			private DateTime m_NextDetectHidden;
			
			private bool m_bDetectHidden;

			public AITimer( BaseAI owner ) : base( TimeSpan.FromSeconds( owner.m_Mobile.CurrentSpeed ), TimeSpan.FromSeconds( owner.m_Mobile.CurrentSpeed ) )
			{
				m_Owner = owner;

				m_bDetectHidden = false;
				m_NextDetectHidden = DateTime.Now;

				Priority = TimerPriority.FiftyMS;				
			}

			protected override void OnTick()
			{
				if ( m_Owner.m_Mobile.Deleted )
				{
					Stop();
					return;
				}

				if ( !m_Owner.Think() )
				{
					Stop();
					return;
				}

				//
				// Just that I dont want to do Detect Hiden for now
				// I'll add it later
				//
				/*
				if (m_bDetectHidden && DateTime.Now > m_NextDetectHidden)
				{
					m_Owner.DetectHiden();
					m_NextDetectHidden = DateTime.Now + TimeSpan.FromSeconds( 0.2 * (double)(150-m_Owner.m_Mobile.Int) );
				}
				else
				{
					m_bDetectHidden = m_Owner.m_Mobile.Skills[SkillName.DetectHidden].Value > 0;
				}
				*/
			}
		}
	}
}