using System;
using System.Collections;
using Server.Targeting;
using Server.Network;

//
// This is a first simple AI
//
//

namespace Server.Mobiles
{
	public class MeeleeAI : BaseAI
	{
		private enum MeeleeState
		{
            MeeleeWander,	// Act normaly
            MeeleeCombat,	// Have a combatant
			MeeleeGuard,	// Just lost a combatant so is in guard
			MeeleeFlee		// Try to flee
		}

		private MeeleeState m_iState;
		private DateTime m_NextStopGuard;

		public MeeleeAI(BaseCreature m) : base (m)
		{
			m_iState = MeeleeState.MeeleeWander;
		}

		public override bool Think()
		{
			if ( m_Mobile.Deleted )
				return false;

			switch (m_iState)
			{
				case MeeleeState.MeeleeWander:
					return DoWander();

				case MeeleeState.MeeleeCombat:
					return DoCombat();

				case MeeleeState.MeeleeGuard:
					return DoGuard();

				case MeeleeState.MeeleeFlee:
					return DoFlee();

				default:
					return false;
			}
		}

		public virtual bool DoWander()
		{
			m_Mobile.DebugSay( "I have no combatant" );

			if ( AquireFocusMob( m_Mobile.RangePerception, AcquireType.Closest, false, false, true ) )
			{
				m_Mobile.DebugSay( "I have detected {0}, attacking", m_Mobile.FocusMob.Name );

				m_Mobile.Combatant = m_Mobile.FocusMob;
				m_Mobile.FocusMob = null;
				m_Mobile.Warmode = true;

				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = MeeleeState.MeeleeCombat;
			}
			else
			{
				m_Mobile.Warmode = false;

				DoWalkRandomSlowly( 5, 1 );
			}

			return true;
		}

		public virtual bool DoCombat()
		{
			Mobile combatant = m_Mobile.Combatant;
			m_Mobile.Warmode = true;

			if ( combatant == null || combatant.Deleted || combatant.Map != m_Mobile.Map )
			{
				m_Mobile.DebugSay( "My combatant is gone, so my guard is up" );

				EnterGuardMode( 5.0, true );

				return true;
			}

			if ( DoWalkMobileRange( combatant, 1, true, m_Mobile.RangeFight, m_Mobile.RangeFight ) )
			{
				m_Mobile.Direction = m_Mobile.GetDirectionTo( combatant );
			}
			else
			{
				if ( m_Mobile.GetDistanceToSqrt( combatant ) > m_Mobile.RangePerception + 1 )
				{
					m_Mobile.DebugSay( "I cannot find {0}, so my guard is up", combatant.Name );

					EnterGuardMode( 10.0, true );

					return true;
				}
				else
				{
					m_Mobile.DebugSay( "I should be closer to {0}", combatant.Name );

					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				}
			}

			if ( m_Mobile.Hits < m_Mobile.HitsMax * 20/100 )
			{
				// We are low on health, should we flee?

				bool flee = false;

				if ( m_Mobile.Hits < combatant.Hits )
				{
					// We are more hurt than them

					int diff = combatant.Hits - m_Mobile.Hits;

					flee = ( Utility.Random( 0, 100 ) > (10 + diff) ); // (10 + diff)% chance to flee
				}
				else
				{
					flee = Utility.Random( 0, 100 ) > 10; // 10% chance to flee
				}

				if ( flee )
				{
					m_Mobile.DebugSay( "I am going to flee from {0}", combatant.Name );

					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_iState = MeeleeState.MeeleeFlee;
				}
			}

			return true;
		}

		public virtual bool DoGuard()
		{
			if ( DateTime.Now < m_NextStopGuard )
			{
				m_Mobile.DebugSay( "I am on guard" );
				m_Mobile.Warmode = true;

				m_Mobile.Turn( Utility.Random( -1, 3 ) );

				if ( AquireFocusMob(m_Mobile.RangePerception, AcquireType.Closest, false, false, true ) )
				{
					m_Mobile.DebugSay( "I have detected {0}, attacking", m_Mobile.FocusMob.Name );

					m_Mobile.Combatant = m_Mobile.FocusMob;
					m_Mobile.FocusMob = null;

					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_iState = MeeleeState.MeeleeCombat;
				}
			}
			else
			{
				m_Mobile.DebugSay( "I am relaxing" );

				m_Mobile.Combatant = null;
				m_Mobile.FocusMob = null;
				m_Mobile.Warmode = false;

				m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
				m_iState = MeeleeState.MeeleeWander;
			}

			return true;
		}

		public virtual bool DoFlee()
		{
			m_Mobile.Warmode = true;

			Mobile combatant = m_Mobile.Combatant;

			if ( combatant == null || combatant.Deleted || combatant.Map != m_Mobile.Map )
			{
				m_Mobile.DebugSay( "My combatant is gone, so my guard is up, but I will not flee" );

				EnterGuardMode( 10.0, true );
			}
			else if ( DoWalkMobileRange( combatant, 1, true, m_Mobile.RangePerception * 2, m_Mobile.RangePerception * 3 ) )
			{
				m_Mobile.DebugSay( "I have fled, but my guard is up" );

				EnterGuardMode( 15.0, true );
			}
			else if ( m_Mobile.Hits > m_Mobile.HitsMax/2 )
			{
				m_Mobile.DebugSay( "I am stronger now, so I will continue fighting" );

				m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
				m_iState = MeeleeState.MeeleeCombat;
			}
			else
			{
				m_Mobile.DebugSay( "I am afraid of {0}", combatant.Name );
			}

			return true;
		}

		public void EnterGuardMode( double duration, bool clearCombatant )
		{
			if ( clearCombatant )
				m_Mobile.Combatant = null;

			m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
			m_iState = MeeleeState.MeeleeGuard;

			m_NextStopGuard = DateTime.Now + TimeSpan.FromSeconds( duration );
		}

/*		public virtual bool DoPanic()
		{
			m_Mobile.DebugSay("*panic*");
			DoWalkRandomNonStop( 2, 4 );

			if ( m_Mobile.Hits > m_Mobile.HitsMax/2 ) // at 50% health we go to guard mode
			{
				m_iState = MeeleeState.MeeleeGuard;
				m_NextStopGuard = DateTime.Now + TimeSpan.FromSeconds( 10 );
			}

			return true;
		}*/
	}
}

// Archive : Find another fighter
/*
if (AquireFocusMobClosest(m_Mobile.RangePerception))
{
	if (m_Mobile.FocusMob != m_Mobile.Combatant)
	{
		if (m_Mobile.FocusMob != null)
		{
			m_Mobile.DebugSay(m_Mobile.FocusMob.Name + " is closer than " + m_Mobile.Combatant.Name);

			if (Utility.Random(0, 10) >= 5)
			{
				m_Mobile.DebugSay("I have keeped my combatant " + m_Mobile.Combatant.Name);
			}
			else
			{
				m_Mobile.Combatant = m_Mobile.FocusMob;
				m_Mobile.DebugSay("I have changed my combatant to" + m_Mobile.Combatant.Name);
			}
		}
	}
}
*/