using System;
using System.Collections;
using Server.Targeting;
using Server.Network;

namespace Server.Mobiles
{
	/// <summary>
	/// Summary description for MobileAI.
	/// </summary>
	/// 
	
	/*
	 * Not sure to be implemented
	 * 
	public enum StatusType
	{
		Normal,
		Tamed,
		Panicked,
		Pacified,	// Under the PeaceMaking skill 
		Provocated	// Unser the Profocation skill
	}
	*/

	/*
	 * Not used for now will replace AcquireType, to have
	 * a generic system for combat personnality
	 * 
	public enum CombatType
	{
		Random,
		Closest,
		Farthest,
		Weakest,
		Strongest,
	}
	*/

	public class BaseCreature : Mobile
	{
		private BaseAI	m_AI;					// THE AI
		
		private AIType	m_CurrentAI;			// The current AI
		private AIType	m_DefaultAI;			// The default AI

		//private CombatType	m_CurrentCombatMode;	// The current combat type
		//private CombatType	m_DefaultCombatMode;	// The default combat type

		private Mobile	m_FocusMob;				// Use focus mob instead of combatant, maybe we don't whan to fight

		private int		m_iRangePerception;		// The view area
		private int		m_iRangeFight;			// The fight distance
       
		private bool	m_bDebugAI;				// Show debug AI messages

		private int		m_iFaction;				// Monster Faction

        private double	m_dActiveSpeed;			// Timer speed when active
		private double	m_dPassiveSpeed;		// Timer speed when not active
		private double	m_dCurrentSpeed;		// The current speed, lets say it could be changed by something;

		private Point3D m_pHome;				// The home position of the creature, used by some AI

		//private Mobile	m_ControlerMod;			// A mobile controling this mobile, 

		public BaseCreature(AIType aiCurrent, AIType aiDefault, int iRangePerception, int iRangeFight, int iFaction, double dActiveSpeed, double dPassiveSpeed)
		{
			m_CurrentAI = aiDefault;
			m_DefaultAI = aiCurrent;

			//m_DefaultCombatMode = combatType;
			//m_CurrentCombatMode = combatType;

			m_iRangePerception = iRangePerception;
			m_iRangeFight = iRangeFight;

			m_iFaction = iFaction;

			m_dActiveSpeed = dActiveSpeed;
			m_dPassiveSpeed = dPassiveSpeed;
			
			m_dCurrentSpeed = dPassiveSpeed;

			m_bDebugAI = false;

			m_pHome.X = Location.X; // NOT WORKING!
			m_pHome.Y = Location.Y; // NOT WORKING!
			m_pHome.Z = Location.Z; // NOT WORKING!

			ChangeAIType(AI);
		}

		public BaseCreature( Serial serial ) : base( serial )
		{
			m_bDebugAI = false;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( (int)m_CurrentAI );
			writer.Write( (int)m_DefaultAI );

			writer.Write( (int)m_iRangePerception );
			writer.Write( (int)m_iRangeFight );

			writer.Write( (int)m_iFaction );

			writer.Write( (double)m_dActiveSpeed );
			writer.Write( (double)m_dPassiveSpeed );
			writer.Write( (double)m_dCurrentSpeed );

			writer.Write( (int) m_pHome.X );
			writer.Write( (int) m_pHome.Y );
			writer.Write( (int) m_pHome.Z );

			// Version 1
			//writer.Write( (int) m_DefaultCombatMode );
			//writer.Write( (int) m_CurrentCombatMode );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			m_CurrentAI = (AIType)reader.ReadInt();
			m_DefaultAI = (AIType)reader.ReadInt();

			m_iRangePerception = reader.ReadInt();
			m_iRangeFight = reader.ReadInt();

			m_iFaction = reader.ReadInt();

			m_dActiveSpeed = reader.ReadDouble();
			m_dPassiveSpeed = reader.ReadDouble();
			m_dCurrentSpeed = reader.ReadDouble();

			m_pHome.X = reader.ReadInt();
			m_pHome.Y = reader.ReadInt();
			m_pHome.Z = reader.ReadInt();

			//if (version >= 1)
			//{
				//m_DefaultCombatMode = reader.ReadInt();
				//m_CurrentCombatMode = reader.ReadInt();
			//}

			ChangeAIType(m_CurrentAI);
		}

		public virtual void ChangeAIType(AIType NewAI)
		{
			if ( m_AI != null )
			{
				m_AI.m_Timer.Stop();
			}
			m_AI = null;

			/*
			 * Realy don't like this way of doing things
			 * Should be changed of somethig else
			 */ 
			switch (NewAI)
			{
				case AIType.AI_Meelee:
					m_AI = new MeeleeAI(this);
					break;
				case AIType.AI_Animal:
					m_AI = new AnimalAI(this);
					break;
				case AIType.AI_Berseck:
					m_AI = new BerseckAI(this);
					break;
				case AIType.AI_Archer:
					m_AI = new ArcherAI(this);
					break;
				case AIType.AI_Healer:
					m_AI = new HealerAI(this);
					break;
			}
		}

		public virtual void ChangeAIToDefault()
		{
           ChangeAIType(m_DefaultAI);
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public AIType AI
		{
			get
			{
				return m_CurrentAI;
			}
			set
			{
				m_CurrentAI = value;

				if (m_CurrentAI == AIType.AI_Use_Default)
				{
					m_CurrentAI = m_DefaultAI;
				}
				
				ChangeAIType(m_CurrentAI);
			}
		}

		[CommandProperty( AccessLevel.Administrator )]
		public bool Debug
		{
			get
			{
				return m_bDebugAI;
			}
			set
			{
				m_bDebugAI = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int Faction
		{
			get
			{
				return m_iFaction;
			}
			set
			{
				m_iFaction = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile FocusMob
		{
			get
			{
				return m_FocusMob;
			}
			set
			{
				m_FocusMob = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int RangePerception
		{
			get
			{
				return m_iRangePerception;
			}
			set
			{
				m_iRangePerception = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int RangeFight
		{
			get
			{
				return m_iRangeFight;
			}
			set
			{
				m_iRangeFight = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public double ActiveSpeed
		{
			get
			{
				return m_dActiveSpeed;
			}
			set
			{
				m_dActiveSpeed = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public double PassiveSpeed
		{
			get
			{
				return m_dPassiveSpeed;
			}
			set
			{
				m_dPassiveSpeed = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public double CurrentSpeed
		{
			get
			{
				return m_dCurrentSpeed;
			}
			set
			{
				m_dCurrentSpeed = value;

				if (m_AI != null)
				{
					m_AI.OnCurrentSpeedChanged();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Point3D Home
		{
			get
			{
				return m_pHome;
			}
			set
			{
				m_pHome = value;
			}
		}

		public override void OnAfterDelete()
		{
			if ( m_AI != null )
			{
				if ( m_AI.m_Timer != null )
					m_AI.m_Timer.Stop();

				m_AI = null;
			}

			FocusMob = null;

			base.OnAfterDelete();
		}

		public void DebugSay( string text )
		{
			if ( m_bDebugAI )
				this.PublicOverheadMessage( MessageType.Regular, 41, false, text );
		}

		public void DebugSay( string format, params object[] args )
		{
			if ( m_bDebugAI )
				this.PublicOverheadMessage( MessageType.Regular, 41, false, String.Format( format, args ) );
		}

		/*
		 * Will need to be givent a better name
		 * 
		 * This function can be overriden.. so a "Strongest" mobile, can have a different definition depending
		 * on who check for value
		 * -Could add a AcquireType.Prefered
		 * 
		 */
		public virtual double GetValueFrom( Mobile m, AcquireType acqType, bool bPlayerOnly )
		{
			if ( ( bPlayerOnly && m.Player ) ||  !bPlayerOnly )
			{
				switch( acqType )
				{
					case AcquireType.Strongest : 
						return (m.Skills[SkillName.Tactics].Value + m.Str); //returns strongest mobile

					case AcquireType.Weakest : 
						return -m.Hits; // returns weakest mobile

					case AcquireType.MageStrongest : 
						return (m.Skills[SkillName.Magery].Value + m.Str);	//returns strongest mage
					default : 
						return -GetDistanceToSqrt( m ); // returns closest mobile
				}
			}
			else
			{
				return double.MinValue;
			}
		}

		// Turn, - for let, + for right
		// Basic for now, need works
		public virtual void Turn(int iTurnSteps)
		{
			int v = (int)Direction;

			Direction = (Direction)((((v & 0x7) + iTurnSteps) & 0x7) | (v & 0x80));
			/*Direction finalDir = Direction;
					
			if ( iTurnSteps > 0 )
			{
				for (int i=0; i<iTurnSteps; i++)
				{
					if ( finalDir == Direction.Up || finalDir == ( Direction.Up|Direction.Running ) )
					{
						finalDir = Direction.North;
					}
					else if ( finalDir == Direction.North || finalDir == ( Direction.North|Direction.Running ) )
					{
						finalDir = Direction.Left;
					}
					else if ( finalDir == Direction.Left || finalDir == ( Direction.Left|Direction.Running ) )
					{
						finalDir = Direction.West;
					}					
					else if ( finalDir == Direction.West || finalDir == ( Direction.West|Direction.Running ) )
					{
						finalDir = Direction.Down;
					}					
					else if ( finalDir == Direction.Down || finalDir == ( Direction.Down|Direction.Running ) )
					{
						finalDir = Direction.South;
					}					
					else if ( finalDir == Direction.South || finalDir == ( Direction.South|Direction.Running ) )
					{
						finalDir = Direction.Right;
					}					
					else if ( finalDir == Direction.Right || finalDir == ( Direction.Right|Direction.Running ) )
					{
						finalDir = Direction.East;
					}					
					else if ( finalDir == Direction.East || finalDir == ( Direction.East|Direction.Running ) )
					{
						finalDir = Direction.Up;
					}
				}
			}
			else if ( iTurnSteps < 0 )
			{
				for (int i=iTurnSteps; i>0; i++)
				{
					if ( finalDir == Direction.Up || finalDir == ( Direction.Up|Direction.Running ) )
					{
						finalDir = Direction.East;
					}
					else if ( finalDir == Direction.North || finalDir == ( Direction.North|Direction.Running ) )
					{
						finalDir = Direction.Up;
					}
					else if ( finalDir == Direction.Left || finalDir == ( Direction.Left|Direction.Running ) )
					{
						finalDir = Direction.North;
					}					
					else if ( finalDir == Direction.West || finalDir == ( Direction.West|Direction.Running ) )
					{
						finalDir = Direction.Left;
					}					
					else if ( finalDir == Direction.Down || finalDir == ( Direction.Down|Direction.Running ) )
					{
						finalDir = Direction.West;
					}					
					else if ( finalDir == Direction.South || finalDir == ( Direction.South|Direction.Running ) )
					{
						finalDir = Direction.Down;
					}					
					else if ( finalDir == Direction.Right || finalDir == ( Direction.Right|Direction.Running ) )
					{
						finalDir = Direction.South;
					}					
					else if ( finalDir == Direction.East || finalDir == ( Direction.East|Direction.Running ) )
					{
						finalDir = Direction.Right;
					}
				}
			}

			SetDirection( finalDir );*/
		}

		public bool IsHurt()
		{
			return ( Hits != HitsMax );
		}

		public double GetHomeDistance()
		{
            return GetDistanceToSqrt( m_pHome );
		}

		/*
		 *  cound of faction, obsolete if use a faction object.
		 */ 
		public virtual int GetFactionSize(int iRange)
		{
			int iCount = 0;
			Map map = this.Map;

			if ( map != null )
			{
				IPooledEnumerable eable = map.GetMobilesInRange( Location, iRange);

				foreach ( Mobile m in eable )
				{
					if (m is BaseCreature)
					{
						if ( ((BaseCreature)m).Faction == Faction )
						{
							if ( !m.Deleted )
							{
								if ( m != this )
								{
									if ( CanSee( m ) )
									{
										iCount++;
									}
								}
							}
						}
					}
				}

				eable.Free();
			}
			
			return iCount;
		}

		// Do my combatant is attaking me??
		public bool IsCombatantAnAgressor()
		{
			if (Combatant != null)
			{
				if (Combatant.Combatant == this)
				{
					return true;
				}
			}
			return false;
		}
	}
}