using System;
using Server.Items;
using Server.Mobiles;

namespace Server.Mobiles
{
	/// <summary>
	/// This is a test creature
	/// You can set its value in game
	/// It die after 5 minutes, so your test server stay clean
	/// Create a macro to help your creation "[add Dummy 1 15 7 -1 0.5 2"
	/// 
	/// A iFaction of negative will set a faction at random
	/// 
	/// Say Kill if you want them to die
	/// 
	/// </summary>
	public class Dummy : BaseCreature
	{
		public Timer	m_Timer;

		[Constructable]
		public Dummy(AIType iAI, int iRangePerception, int iRangeFight, int iFaction, double dActiveSpeed, double dPassiveSpeed) : base(iAI, iAI, iRangePerception, iRangeFight, iFaction, dActiveSpeed, dPassiveSpeed)
		{
			this.Body = 400;
			this.InitStats( 50, 50, 50 );

			this.Skills[SkillName.Archery].Base = 50;
			this.Skills[SkillName.Tactics].Base = 50;
			this.Skills[SkillName.Anatomy].Base = 50;
			this.Skills[SkillName.DetectHidden].Base = 50.0;
			this.Skills[SkillName.Magery].Base = 50.0;
			this.Skills[SkillName.Swords].Base = 50.0;
			this.Skills[SkillName.Meditation].Base = 50.0;
			this.Skills[SkillName.MagicResist].Base = 50.0;

			if (iFaction < 0)
			{
				Faction = Utility.Random(-iFaction);
			}

			int iHue = 20 + Faction * 40;

			if (iAI == AIType.AI_Meelee)
			{
				this.Name = "Fighter";

				Longsword swd = new Longsword();

				swd.Movable = false;
				swd.Newbied = true;
				swd.Crafter = this;
				swd.Quality = WeaponQuality.Low;
				swd.Hue = iHue;

				AddItem( swd );
			}

			if (iAI == AIType.AI_Animal)
			{
				this.Name = "Animal";
			}

			if (iAI == AIType.AI_Archer)
			{
				this.Name = "Archer";

				Bow bow = new Bow();

				bow.Movable = false;
				bow.Newbied = true;
				bow.Crafter = this;
				bow.Quality = WeaponQuality.Low;
				bow.Hue = iHue;

				AddItem( bow );
			}

			if (iAI == AIType.AI_Healer)
			{
				this.Name = "Healer";

				Mace swd = new Mace();

				swd.Movable = false;
				swd.Newbied = true;
				swd.Crafter = this;
				swd.Quality = WeaponQuality.Low;
				swd.Hue = iHue;

				AddItem( swd );
			}

			SkullCap cap = new SkullCap();
			cap.Hue = iHue;
			AddItem(cap);

			ShortPants pants = new ShortPants();
			pants.Hue = iHue;
			AddItem(pants);

			LeatherGloves glv = new LeatherGloves();
			glv.Hue = iHue;
			AddItem(glv);

			Container pack = new Backpack();

			pack.Movable = false;
			pack.Newbied = true;

			Arrow arrows = new Arrow( 20 );
			arrows.Hue = iHue;

			arrows.Newbied = true;

			pack.DropItem( arrows );
			pack.DropItem( new Gold( 250, 500 ) );

			AddItem( pack );

			m_Timer = new AutokillTimer(this);
			m_Timer.Start();
		}

		public Dummy( Serial serial ) : base( serial )
		{
			m_Timer = new AutokillTimer(this);
			m_Timer.Start();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void OnSpeech( SpeechEventArgs e )
		{
			if (e.Mobile.AccessLevel >= AccessLevel.GameMaster)
			{
				if (e.Speech == "kill")
				{
					m_Timer.Stop();
					m_Timer.Delay = TimeSpan.FromSeconds( Utility.Random(1, 5) );
					m_Timer.Start();
					
				}
			}
		}

		private class AutokillTimer : Timer
		{
			private Dummy m_Owner;

			public AutokillTimer( Dummy owner ) : base( TimeSpan.FromMinutes(5.0) )
			{
				m_Owner = owner;
			}

			protected override void OnTick()
			{
				m_Owner.Kill();
				Stop();
			}
		}
	}
}
