using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Misc;

namespace Server.Mobiles
{
	public class Armorer : BaseVendor
	{
		private IShopSellInfo[] m_ArmorSellInfo = new IShopSellInfo[]{ new ArmorSellInfo() };
		private IBuyItemInfo[] m_ArmorBuyInfo = new IBuyItemInfo[]
		{
			//---------------------Type---------------------Price------Amount--------------ID---Hue
			new GenericBuyInfo( typeof( LeatherArms ),		  80, Utility.Random( 5, 10 ), 0x13CD, 0 ), 
			new GenericBuyInfo( typeof( LeatherChest ),		 101, Utility.Random( 5, 10 ), 0x13CC, 0 ), 
			new GenericBuyInfo( typeof( LeatherGloves ),	  60, Utility.Random( 5, 10 ), 0x13C6, 0 ), 
			new GenericBuyInfo( typeof( LeatherGorget ),	  74, Utility.Random( 5, 10 ), 0x13C7, 0 ), 
			new GenericBuyInfo( typeof( LeatherLegs ),		  80, Utility.Random( 5, 10 ), 0x13cb, 0 ), 

			new GenericBuyInfo( typeof( LeatherCap ),		  75, Utility.Random( 5, 10 ), 0x1DB9, 0 ), 
			new GenericBuyInfo( typeof( FemaleLeatherChest ),116, Utility.Random( 5, 10 ), 0x1C06, 0 ), 
			new GenericBuyInfo( typeof( LeatherBustierArms ), 97, Utility.Random( 5, 10 ), 0x1C0A, 0 ), 
			new GenericBuyInfo( typeof( LeatherShorts ),	  86, Utility.Random( 5, 10 ), 0x1C00, 0 ), 
			new GenericBuyInfo( typeof( LeatherSkirt ),		  87, Utility.Random( 5, 10 ), 0x1C08, 0 ), 

			new GenericBuyInfo( typeof( StuddedArms ),		  87, Utility.Random( 5, 10 ), 0x13DC, 0 ), 
			new GenericBuyInfo( typeof( StuddedChest ),		 128, Utility.Random( 5, 10 ), 0x13DB, 0 ), 
			new GenericBuyInfo( typeof( StuddedGloves ),	  79, Utility.Random( 5, 10 ), 0x13D5, 0 ), 
			new GenericBuyInfo( typeof( StuddedGorget ),	  73, Utility.Random( 5, 10 ), 0x13D6, 0 ), 
			new GenericBuyInfo( typeof( StuddedLegs ),		 103, Utility.Random( 5, 10 ), 0x13DA, 0 ), 
	
			new GenericBuyInfo( typeof( FemaleStuddedChest ),142, Utility.Random( 5, 10 ), 0x1C02, 0 ), 
			new GenericBuyInfo( typeof( StuddedBustierArms ),120, Utility.Random( 5, 10 ), 0x1c0c, 0 ), 

			new GenericBuyInfo( typeof( RingmailArms ),		 122, Utility.Random( 5, 10 ), 0x13EE, 0 ), 
			new GenericBuyInfo( typeof( RingmailChest ),	 218, Utility.Random( 5, 10 ), 0x13ec, 0 ), 
			new GenericBuyInfo( typeof( RingmailGloves ),	 122, Utility.Random( 5, 10 ), 0x13eb, 0 ), 
			new GenericBuyInfo( typeof( RingmailLegs ),		 147, Utility.Random( 5, 10 ), 0x13F0, 0 ), 

			new GenericBuyInfo( typeof( ChainChest ),		 207, Utility.Random( 5, 10 ), 0x13BF, 0 ), 
			new GenericBuyInfo( typeof( ChainLegs ),		 166, Utility.Random( 5, 10 ), 0x13BE, 0 ), 

			new GenericBuyInfo( typeof( ChainCoif ),		 130, Utility.Random( 5, 10 ), 0x13BB, 0 ), 

			new GenericBuyInfo( typeof( PlateArms ),		 181, Utility.Random( 5, 10 ), 0x1410, 0 ), 
			new GenericBuyInfo( typeof( PlateChest ),		 273, Utility.Random( 5, 10 ), 0x1415, 0 ), 
			new GenericBuyInfo( typeof( PlateGloves ),		 145, Utility.Random( 5, 10 ), 0x1414, 0 ), 
			new GenericBuyInfo( typeof( PlateGorget ),		 124, Utility.Random( 5, 10 ), 0x1413, 0 ), 
			new GenericBuyInfo( typeof( PlateLegs ),		 218, Utility.Random( 5, 10 ), 0x1411, 0 ), 

			new GenericBuyInfo( typeof( PlateHelm ),		 170, Utility.Random( 5, 10 ), 0x1412, 0 ), 
			new GenericBuyInfo( typeof( FemalePlateChest ),	 245, Utility.Random( 5, 10 ), 0x1C04, 0 ), 
		
			new GenericBuyInfo( typeof( Bascinet ),			 127, Utility.Random( 5, 10 ), 0x140C, 0 ), 
			new GenericBuyInfo( typeof( CloseHelm ),		 145, Utility.Random( 5, 10 ), 0x1408, 0 ), 
			new GenericBuyInfo( typeof( Helmet ),			 116, Utility.Random( 5, 10 ), 0x140A, 0 ), 
			new GenericBuyInfo( typeof( NorseHelm ),		 145, Utility.Random( 5, 10 ), 0x140E, 0 ), 
		};

		[Constructable()]
		public Armorer() : base( "the armorer" )
		{
			InitStats( 100, 100, 25 );

			SpeechHue = Utility.RandomDyedHue();
			Hue = Utility.RandomSkinHue();

			if ( this.Female = Utility.RandomBool() )
			{
				this.Body = 0x191;
				this.Name = NameList.RandomFemaleName();
			}
			else
			{
				this.Body = 0x190;
				this.Name = NameList.RandomMaleName();
			}

			AddItem( new FancyShirt( Utility.RandomNeutralHue() ) );
			AddItem( new ShortPants( Utility.RandomNeutralHue() ) );
			AddItem( new Boots( Utility.RandomNeutralHue() ) );

			Item hair = new Item( Utility.RandomList( 0x203B, 0x2049, 0x2048, 0x204A ) );

			hair.Hue = Utility.RandomNondyedHue();
			hair.Layer = Layer.Hair;
			hair.Movable = false;
			hair.Newbied = true;

			AddItem( hair );

			Container pack = new Backpack();

			pack.DropItem( new Gold( 1000, 2000 ) );

			pack.Movable = false;
			pack.Newbied = true;

			AddItem( pack );
		}

		public Armorer( Serial serial ) : base( serial )
		{
		}

		public override IShopSellInfo[] GetSellInfo()
		{
			return m_ArmorSellInfo;
		}

		public override IBuyItemInfo[] GetBuyInfo()
		{
			return m_ArmorBuyInfo;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class ArmorSellInfo : IShopSellInfo
	{
		private static Type[] m_MyTypes = new Type[]
		{
			typeof ( LeatherArms ),
			typeof ( LeatherChest ),
			typeof ( LeatherGloves ),
			typeof ( LeatherGorget ),
			typeof ( LeatherLegs ),

			typeof ( LeatherCap ),
			typeof ( FemaleLeatherChest ),
			typeof ( LeatherBustierArms ),
			typeof ( LeatherShorts ),
			typeof ( LeatherSkirt ),

			typeof ( StuddedArms ),
			typeof ( StuddedChest ),
			typeof ( StuddedGloves ),
			typeof ( StuddedGorget ),
			typeof ( StuddedLegs ),

			typeof ( FemaleStuddedChest ),
			typeof ( StuddedBustierArms ),

			typeof ( RingmailArms ),
			typeof ( RingmailChest ),
			typeof ( RingmailGloves ),
			typeof ( RingmailLegs ),

			typeof ( ChainChest ),
			typeof ( ChainLegs ),

			typeof ( ChainCoif ),

			typeof ( PlateArms ),
			typeof ( PlateChest ),
			typeof ( PlateGloves ),
			typeof ( PlateGorget ),
			typeof ( PlateLegs ),

			typeof ( PlateHelm ),
			typeof ( FemalePlateChest ),
		
			typeof ( Bascinet ),
			typeof ( CloseHelm ),
			typeof ( Helmet ),
			typeof ( NorseHelm )
	};

		public ArmorSellInfo()
		{
		}

		public int GetSellPriceFor( Item item )
		{
			int price = 0;
			
			if ( item is LeatherArms ) price = 40;
			else if ( item is LeatherChest ) price = 52;
			else if ( item is LeatherGloves ) price = 30;
			else if ( item is LeatherGorget ) price = 37;
			else if ( item is LeatherLegs ) price = 40;

			else if ( item is LeatherCap ) price = 31;
			else if ( item is FemaleLeatherChest ) price = 58;
			else if ( item is LeatherBustierArms ) price = 48;
			else if ( item is LeatherShorts ) price = 43;
			else if ( item is LeatherSkirt ) price = 43;

			else if ( item is StuddedArms ) price = 43;
			else if ( item is StuddedChest ) price = 64;
			else if ( item is StuddedGloves ) price = 39;
			else if ( item is StuddedGorget ) price = 36;
			else if ( item is StuddedLegs ) price = 51;

			else if ( item is FemaleStuddedChest ) price = 71;
			else if ( item is StuddedBustierArms ) price = 60;

			else if ( item is RingmailArms ) price = 61;
			else if ( item is RingmailChest ) price = 109;
			else if ( item is RingmailGloves ) price = 61;
			else if ( item is RingmailLegs ) price = 73;

			else if ( item is ChainChest ) price = 103;
			else if ( item is ChainLegs ) price = 83;

			else if ( item is ChainCoif ) price = 65;

			else if ( item is PlateArms ) price = 90;
			else if ( item is PlateChest ) price = 136;
			else if ( item is PlateGloves ) price = 72;
			else if ( item is PlateGorget ) price = 70;
			else if ( item is PlateLegs ) price = 109;

			else if ( item is PlateHelm ) price = 85;
			else if ( item is FemalePlateChest ) price = 122;

			else if ( item is Bascinet ) price = 46;
			else if ( item is CloseHelm ) price = 72;
			else if ( item is Helmet ) price = 58;
			else if ( item is NorseHelm ) price = 53;

			if ( item is BaseArmor )
			{
				BaseArmor armor = (BaseArmor)item;

				if ( armor.Quality == ArmorQuality.Low )
					price = (int)( price * 0.60 );
				else if ( armor.Quality == ArmorQuality.Exceptional )
					price = (int)( price * 1.25 );

				price += 100 * (int)armor.Durability;

				price += 100 * (int)armor.ProtectionLevel;

				if ( price < 1 )
					price = 1;
			}

			return price;
		}

		public int GetBuyPriceFor( Item item )
		{
			return (int)( 1.90 * GetSellPriceFor( item ) );
		}

		public Type[] Types
		{
			get
			{
				return m_MyTypes;
			}
		}

		public string GetNameFor( Item item )
		{
			if ( item.Name != null )
				return item.Name;
			else
				return (1020000 + (item.ItemID & 0x3FFF)).ToString();
		}

		public bool IsSellable( Item item )
		{
			if ( item.Hue != 0 )
				return false;

			return IsInList( item.GetType() );
		}
 
		public bool IsResellable( Item item )
		{
			if ( item.Hue != 0 )
				return false;

			return IsInList( item.GetType() );
		}

		public bool IsInList( Type type )
		{
			for ( int i = 0; i < m_MyTypes.Length; i++ )
				if ( m_MyTypes[i] == type )
					return true;

			return false;
		}
	}
}