using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Misc;

namespace Server.Mobiles
{
	public class Tailor : BaseVendor
	{
		private IShopSellInfo[] m_TailorSellInfo = new IShopSellInfo[]{ new TailorSellInfo() };
		private IBuyItemInfo[] m_TailorBuyInfo = new IBuyItemInfo[]
		{
			//---------------------Type---------------------Price------Amount--------------ID---Hue
			new GenericBuyInfo( typeof( Scissors ),			  13, Utility.Random( 5, 10 ), 0xF9F, 0 ), 
			new GenericBuyInfo( typeof( SewingKit ),		   3, Utility.Random( 5, 10 ), 0xF9D, 0 ), 
			new GenericBuyInfo( typeof( Dyes ),				   8, Utility.Random( 5, 10 ), 0xFA9, 0 ), 
			new GenericBuyInfo( typeof( DyeTub ),			   9, Utility.Random( 5, 10 ), 0xFAB, 0 ), 

			new GenericBuyInfo( typeof( FancyShirt ),		  55, Utility.Random( 5, 10 ), 0x1EFD, 0 ), 
			new GenericBuyInfo( typeof( Shirt ),			  37, Utility.Random( 5, 10 ), 0x1517, 0 ), 

			new GenericBuyInfo( typeof( ShortPants ),		  24, Utility.Random( 5, 10 ), 0x152E, 0 ), 
			new GenericBuyInfo( typeof( LongPants ),		  34, Utility.Random( 5, 10 ), 0x1539, 0 ), 

			new GenericBuyInfo( typeof( Cloak ),			  43, Utility.Random( 5, 10 ), 0x1515, 0 ), 
			new GenericBuyInfo( typeof( FancyDress ),		  25, Utility.Random( 5, 10 ), 0x1EFF, 0 ), 
			new GenericBuyInfo( typeof( Robe ),				  68, Utility.Random( 5, 10 ), 0x1F03, 0 ), 
			new GenericBuyInfo( typeof( PlainDress ),		  56, Utility.Random( 5, 10 ), 0x1F01, 0 ), 

			new GenericBuyInfo( typeof( Skirt ),			  33, Utility.Random( 5, 10 ), 0x1516, 0 ), 
			new GenericBuyInfo( typeof( Kilt ),				  31, Utility.Random( 5, 10 ), 0x1537, 0 ), 

			new GenericBuyInfo( typeof( FullApron ),		  26, Utility.Random( 5, 10 ), 0x153d, 0 ), 
			new GenericBuyInfo( typeof( HalfApron ),		  26, Utility.Random( 5, 10 ), 0x153b, 0 ), 

			new GenericBuyInfo( typeof( Doublet ),			  25, Utility.Random( 5, 10 ), 0x1F7B, 0 ), 
			new GenericBuyInfo( typeof( Tunic ),			  33, Utility.Random( 5, 10 ), 0x1FA1, 0 ), 
			new GenericBuyInfo( typeof( JesterSuit ),		  51, Utility.Random( 5, 10 ), 0x1F9F, 0 ), 

			new GenericBuyInfo( typeof( JesterHat ),		  31, Utility.Random( 5, 10 ), 0x171C, 0 ), 
			new GenericBuyInfo( typeof( FloppyHat ),		  25, Utility.Random( 5, 10 ), 0x1713, 0 ), 
			new GenericBuyInfo( typeof( WideBrimHat ),		  26, Utility.Random( 5, 10 ), 0x1714, 0 ), 
			new GenericBuyInfo( typeof( Cap ),				  27, Utility.Random( 5, 10 ), 0x1715, 0 ), 
			new GenericBuyInfo( typeof( SkullCap ),			  12, Utility.Random( 5, 10 ), 0x1544, 0 ), 
			new GenericBuyInfo( typeof( Bandana ),			  14, Utility.Random( 5, 10 ), 0x1540, 0 ), 
			new GenericBuyInfo( typeof( TallStrawHat ),		  26, Utility.Random( 5, 10 ), 0x1716, 0 ), 
			new GenericBuyInfo( typeof( StrawHat ),			  25, Utility.Random( 5, 10 ), 0x1717, 0 ), 
			new GenericBuyInfo( typeof( WizardsHat ),		  30, Utility.Random( 5, 10 ), 0x1718, 0 ), 
			new GenericBuyInfo( typeof( Bonnet ),			  26, Utility.Random( 5, 10 ), 0x1719, 0 ), 
			new GenericBuyInfo( typeof( FeatheredHat ),		  27, Utility.Random( 5, 10 ), 0x171A, 0 ), 
			new GenericBuyInfo( typeof( TricorneHat ),		  26, Utility.Random( 5, 10 ), 0x171B, 0 ), 
		};

		[Constructable()]
		public Tailor() : base( "the tailor" )
		{
			InitStats( 100, 100, 25 );

			SpeechHue = Utility.RandomDyedHue();
			Hue = Utility.RandomSkinHue();

			if ( this.Female = Utility.RandomBool() )
			{
				this.Body = 0x191;
				this.Name = NameList.RandomFemaleName();
			}
			else
			{
				this.Body = 0x190;
				this.Name = NameList.RandomMaleName();
			}

			AddItem( new FancyShirt( Utility.RandomNeutralHue() ) );
			AddItem( new ShortPants( Utility.RandomNeutralHue() ) );
			AddItem( new Boots( Utility.RandomNeutralHue() ) );

			Item hair = new Item( Utility.RandomList( 0x203B, 0x2049, 0x2048, 0x204A ) );

			hair.Hue = Utility.RandomNondyedHue();
			hair.Layer = Layer.Hair;
			hair.Movable = false;
			hair.Newbied = true;

			AddItem( hair );

			Container pack = new Backpack();

			pack.DropItem( new Gold( 1000, 2000 ) );

			pack.Movable = false;
			pack.Newbied = true;

			AddItem( pack );
		}

		public Tailor( Serial serial ) : base( serial )
		{
		}

		public override IShopSellInfo[] GetSellInfo()
		{
			return m_TailorSellInfo;
		}

		public override IBuyItemInfo[] GetBuyInfo()
		{
			return m_TailorBuyInfo;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class TailorSellInfo : IShopSellInfo
	{
		private static Type[] m_MyTypes = new Type[]
		{
			typeof ( Scissors ),
			typeof ( SewingKit ),
			typeof ( Dyes ),
			typeof ( DyeTub ),
			
			typeof ( FancyShirt ),
			typeof ( Shirt ),

			typeof ( ShortPants ),
			typeof ( LongPants ),

			typeof ( Cloak ),
			typeof ( FancyDress ),
			typeof ( Robe ),
			typeof ( PlainDress ),

			typeof ( Skirt ),
			typeof ( Kilt ),

			typeof ( Doublet ),
			typeof ( Tunic ),
			typeof ( JesterSuit ),

			typeof ( FullApron ),
			typeof ( HalfApron ),

			typeof ( JesterHat ),
			typeof ( FloppyHat ),
			typeof ( WideBrimHat ),
			typeof ( Cap ),
			typeof ( SkullCap ),
			typeof ( Bandana ),
			typeof ( TallStrawHat ),
			typeof ( StrawHat ),
			typeof ( WizardsHat ),
			typeof ( Bonnet ),
			typeof ( FeatheredHat ),
			typeof ( TricorneHat )		
		};

		public TailorSellInfo()
		{
		}

		public int GetSellPriceFor( Item item )
		{
			int price = 0;
			
			if ( item is Scissors ) price = 6;
			else if ( item is SewingKit ) price = 1;
			else if ( item is Dyes ) price = 4;
			else if ( item is DyeTub ) price = 4;

			else if ( item is FancyShirt ) price = 27;
			else if ( item is Shirt ) price = 18;

			else if ( item is ShortPants ) price = 12;
			else if ( item is LongPants ) price = 17;

			else if ( item is Cloak ) price = 21;
			else if ( item is FancyDress ) price = 12;
			else if ( item is Robe ) price = 34;
			else if ( item is PlainDress ) price = 28;

			else if ( item is Skirt ) price = 16;
			else if ( item is Kilt ) price = 15;

			else if ( item is Doublet ) price = 12;
			else if ( item is Tunic ) price = 16;
			else if ( item is JesterSuit ) price = 25;

			else if ( item is FullApron ) price = 13;
			else if ( item is HalfApron ) price = 13;

			else if ( item is JesterHat ) price = 15;
			else if ( item is FloppyHat ) price = 12;
			else if ( item is WideBrimHat ) price = 13;
			else if ( item is Cap ) price = 13;
			else if ( item is SkullCap ) price = 6;
			else if ( item is Bandana ) price = 6;
			else if ( item is TallStrawHat ) price = 13;
			else if ( item is StrawHat ) price = 12;
			else if ( item is WizardsHat ) price = 15;
			else if ( item is Bonnet ) price = 13;
			else if ( item is FeatheredHat ) price = 13;
			else if ( item is TricorneHat ) price = 13;

			return price;
		}

		public int GetBuyPriceFor( Item item )
		{
			return (int)( 1.90 * GetSellPriceFor( item ) );
		}

		public Type[] Types
		{
			get
			{
				return m_MyTypes;
			}
		}

		public string GetNameFor( Item item )
		{
			if ( item.Name != null )
				return item.Name;
			else
				return (1020000 + (item.ItemID & 0x3FFF)).ToString();
		}

		public bool IsSellable( Item item )
		{
			return IsInList( item.GetType() );
		}
 
		public bool IsResellable( Item item )
		{
			return IsInList( item.GetType() );
		}

		public bool IsInList( Type type )
		{
			for ( int i = 0; i < m_MyTypes.Length; i++ )
				if ( m_MyTypes[i] == type )
					return true;

			return false;
		}
	}
}