using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Misc;

namespace Server.Mobiles
{
	public class Weaponsmith : BaseVendor
	{
		private IShopSellInfo[] m_WeaponSellInfo = new IShopSellInfo[]{ new WeaponSellInfo() };
		private IBuyItemInfo[] m_WeaponBuyInfo = new IBuyItemInfo[]
		{
			//---------------------Type--------------------Price--------Amount-------------ID---Hue
			new GenericBuyInfo( typeof( BattleAxe ),		 38, Utility.Random( 5, 10 ), 0xF47, 0 ), 
			new GenericBuyInfo( typeof( DoubleAxe ),		 32, Utility.Random( 5, 10 ), 0xF4B, 0 ), 
			new GenericBuyInfo( typeof( ExecutionersAxe ),	 38, Utility.Random( 5, 10 ), 0xF45, 0 ), 
			new GenericBuyInfo( typeof( LargeBattleAxe ),	 43, Utility.Random( 5, 10 ), 0x13FB, 0 ),
			new GenericBuyInfo( typeof( Pickaxe ),			 32, Utility.Random( 5, 10 ), 0xE86, 0 ),
			new GenericBuyInfo( typeof( TwoHandedAxe ),		 42, Utility.Random( 5, 10 ), 0x1443, 0 ),
			new GenericBuyInfo( typeof( WarAxe ),			 38, Utility.Random( 5, 10 ), 0x13B0, 0 ),
			new GenericBuyInfo( typeof( Axe ),				 48, Utility.Random( 5, 10 ), 0xF49, 0 ),

			new GenericBuyInfo( typeof( ButcherKnife ),		 21, Utility.Random( 5, 10 ), 0x14F6, 0 ),
			new GenericBuyInfo( typeof( Cleaver ),			 24, Utility.Random( 5, 10 ), 0xEC3, 0 ),
			new GenericBuyInfo( typeof( Dagger ),			 33, Utility.Random( 5, 10 ), 0xF52, 0 ),
			new GenericBuyInfo( typeof( SkinningKnife ),	 26, Utility.Random( 5, 10 ), 0xEC4, 0 ),

			new GenericBuyInfo( typeof( Club ),				 27, Utility.Random( 5, 10 ), 0x13B4, 0 ),
			new GenericBuyInfo( typeof( HammerPick ),		 31, Utility.Random( 5, 10 ), 0x143D, 0 ),
			new GenericBuyInfo( typeof( Mace ),				 38, Utility.Random( 5, 10 ), 0xEC4, 0 ),
			new GenericBuyInfo( typeof( Maul ),				 31, Utility.Random( 5, 10 ), 0x143B, 0 ),
			new GenericBuyInfo( typeof( WarHammer ),		 27, Utility.Random( 5, 10 ), 0x1439, 0 ),
			new GenericBuyInfo( typeof( WarMace ),			 37, Utility.Random( 5, 10 ), 0x1407, 0 ),

			new GenericBuyInfo( typeof( Bardiche ),			 68, Utility.Random( 5, 10 ), 0xF4D, 0 ),
			new GenericBuyInfo( typeof( Halberd ),			 50, Utility.Random( 5, 10 ), 0x143E, 0 ),

			new GenericBuyInfo( typeof( Spear ),			 38, Utility.Random( 5, 10 ), 0xF62, 0 ),
			new GenericBuyInfo( typeof( Pitchfork ),		 25, Utility.Random( 5, 10 ), 0xE87, 0 ),
			new GenericBuyInfo( typeof( ShortSpear ),		 32, Utility.Random( 5, 10 ), 0x1407, 0 ),

			new GenericBuyInfo( typeof( Broadsword ),		 44, Utility.Random( 5, 10 ), 0xF5E, 0 ),
			new GenericBuyInfo( typeof( Cutlass ),			 32, Utility.Random( 5, 10 ), 0x1441, 0 ),
			new GenericBuyInfo( typeof( Katana ),			 42, Utility.Random( 5, 10 ), 0x13FF, 0 ),
			new GenericBuyInfo( typeof( Kryss ),			 42, Utility.Random( 5, 10 ), 0x1401, 0 ),
			new GenericBuyInfo( typeof( Longsword ),		 60, Utility.Random( 5, 10 ), 0xF61, 0 ),
			new GenericBuyInfo( typeof( Scimitar ),			 43, Utility.Random( 5, 10 ), 0x13B6, 0 ),
			new GenericBuyInfo( typeof( ThinLongsword ),	 60, Utility.Random( 5, 10 ), 0x13B8, 0 ),
			new GenericBuyInfo( typeof( VikingSword ),		 66, Utility.Random( 5, 10 ), 0x13B9, 0 )
		};

		[Constructable()]
		public Weaponsmith() : base( "the weaponsmith" )
		{
			InitStats( 100, 100, 25 );

			SpeechHue = Utility.RandomDyedHue();
			Hue = Utility.RandomSkinHue();

			if ( this.Female = Utility.RandomBool() )
			{
				this.Body = 0x191;
				this.Name = NameList.RandomFemaleName();
			}
			else
			{
				this.Body = 0x190;
				this.Name = NameList.RandomMaleName();
			}

			AddItem( new FancyShirt( Utility.RandomNeutralHue() ) );
			AddItem( new ShortPants( Utility.RandomNeutralHue() ) );
			AddItem( new Boots( Utility.RandomNeutralHue() ) );

			Item hair = new Item( Utility.RandomList( 0x203B, 0x2049, 0x2048, 0x204A ) );

			hair.Hue = Utility.RandomNondyedHue();
			hair.Layer = Layer.Hair;
			hair.Movable = false;
			hair.Newbied = true;

			AddItem( hair );

			Container pack = new Backpack();

			pack.DropItem( new Gold( 1000, 2000 ) );

			pack.Movable = false;
			pack.Newbied = true;

			AddItem( pack );
		}

		public Weaponsmith( Serial serial ) : base( serial )
		{
		}

		public override IShopSellInfo[] GetSellInfo()
		{
			return m_WeaponSellInfo;
		}

		public override IBuyItemInfo[] GetBuyInfo()
		{
			return m_WeaponBuyInfo;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class WeaponSellInfo : IShopSellInfo
	{
		private static Type[] m_MyTypes = new Type[]
		{
			typeof ( BattleAxe ),
			typeof ( DoubleAxe ),
			typeof ( ExecutionersAxe ),
			typeof ( LargeBattleAxe ),
			typeof ( Pickaxe ),
			typeof ( TwoHandedAxe ),
			typeof ( WarAxe ),
			typeof ( Axe ),

			typeof ( ButcherKnife ),
			typeof ( Cleaver ),
			typeof ( Dagger ),
			typeof ( SkinningKnife ),

			typeof ( Club ),
			typeof ( HammerPick ),
			typeof ( Mace ),
			typeof ( Maul ),
			typeof ( WarHammer ),
			typeof ( WarMace ),

			typeof ( Bardiche ),
			typeof ( Halberd ),

			typeof ( Spear ),
			typeof ( Pitchfork ),
			typeof ( ShortSpear ),

			typeof ( Broadsword ),
			typeof ( Cutlass ),
			typeof ( Katana ),
			typeof ( Kryss ),
			typeof ( Longsword ),
			typeof ( Scimitar ),
			typeof ( ThinLongsword ),
			typeof ( VikingSword )
		};

		public WeaponSellInfo()
		{
		}

		public int GetSellPriceFor( Item item )
		{
			int price = 0;
			
			if ( item is BattleAxe ) price = 36;
			else if ( item is DoubleAxe ) price = 16;
			else if ( item is ExecutionersAxe ) price = 33;
			else if ( item is LargeBattleAxe ) price = 21;
			else if ( item is Pickaxe ) price = 16;
			else if ( item is TwoHandedAxe ) price = 21;
			else if ( item is WarAxe ) price = 19;
			else if ( item is Axe ) price = 24;

			else if ( item is ButcherKnife ) price = 10;
			else if ( item is Cleaver ) price = 12;
			else if ( item is Dagger ) price = 21;
			else if ( item is SkinningKnife ) price = 13;

			else if ( item is Club ) price = 13;
			else if ( item is HammerPick ) price = 15;
			else if ( item is Mace ) price = 30;
			else if ( item is Maul ) price = 15;
			else if ( item is WarHammer ) price = 13;
			else if ( item is WarMace ) price = 18;

			else if ( item is Bardiche ) price = 34;
			else if ( item is Halberd ) price = 25;

			else if ( item is Spear ) price = 36;
			else if ( item is Pitchfork ) price = 12;
			else if ( item is ShortSpear ) price = 16;

			else if ( item is Broadsword ) price = 22;
			else if ( item is Cutlass ) price = 27;
			else if ( item is Katana ) price = 21;
			else if ( item is Kryss ) price = 21;
			else if ( item is Longsword ) price = 30;
			else if ( item is Scimitar ) price = 21;
			else if ( item is ThinLongsword ) price = 30;
			else if ( item is VikingSword ) price = 33;

			if ( item is BaseWeapon )
			{
				BaseWeapon weapon = (BaseWeapon)item;

				if ( weapon.Quality == WeaponQuality.Low )
					price = (int)( price * 0.60 );
				else if ( weapon.Quality == WeaponQuality.Exceptional )
					price = (int)( price * 1.25 );

				price += 100 * (int)weapon.DurabilityLevel;

				price += 100 * (int)weapon.DamageLevel;

				if ( price < 1 )
					price = 1;
			}

			return price;
		}

		public int GetBuyPriceFor( Item item )
		{
			return (int)( 1.90 * GetSellPriceFor( item ) );
		}

		public Type[] Types
		{
			get
			{
				return m_MyTypes;
			}
		}

		public string GetNameFor( Item item )
		{
			if ( item.Name != null )
				return item.Name;
			else
				return (1020000 + (item.ItemID & 0x3FFF)).ToString();
		}

		public bool IsSellable( Item item )
		{
			return IsInList( item.GetType() );
		}
 
		public bool IsResellable( Item item )
		{
			return IsInList( item.GetType() );
		}

		public bool IsInList( Type type )
		{
			for ( int i = 0; i < m_MyTypes.Length; i++ )
				if ( m_MyTypes[i] == type )
					return true;

			return false;
		}
	}
}