using Server;
using System;
using System.Collections;
using Server.Multis;
using Server.Targeting;

namespace Server.Multis.Deeds
{
	public class HousePlacementTarget : MultiTarget
	{
		private HouseDeed m_Deed;

		public HousePlacementTarget( HouseDeed deed ) : base( deed.MultiID, deed.Offset )
		{
			m_Deed = deed;
		}

		protected override void OnTarget( Mobile from, object o )
		{
			IPoint3D ip = o as IPoint3D;

			if ( ip != null )
			{
				if ( ip is Item )
					ip = ((Item)ip).GetWorldLocation();

				Point3D p = new Point3D( ip );

				if ( Region.Find( new Point3D( p ), from.Map ).AllowHousing( from, p ) )
					m_Deed.OnPlacement( from, p );
				else
					from.SendLocalizedMessage( 501265 ); // Housing can not be created in this area.
			}
		}
	}

	public abstract class HouseDeed : Item
	{
		private int m_MultiID;
		private Point3D m_Offset;

		[CommandProperty( AccessLevel.GameMaster )]
		public int MultiID
		{
			get
			{
				return m_MultiID;
			}
			set
			{
				m_MultiID = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Point3D Offset
		{
			get
			{
				return m_Offset;
			}
			set
			{
				m_Offset = value;
			}
		}

		public HouseDeed( int id, Point3D offset ) : base( 0x14F0 )
		{
			Newbied = true;

			m_MultiID = id & 0x3FFF;
			m_Offset = offset;
		}

		public HouseDeed( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( m_Offset );

			writer.Write( m_MultiID );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
				{
					m_Offset = reader.ReadPoint3D();

					goto case 0;
				}
				case 0:
				{
					m_MultiID = reader.ReadInt();

					break;
				}
			}
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( !IsChildOf( from.Backpack ) )
			{
				from.SendLocalizedMessage( 1042001 ); // That must be in your pack for you to use it.
			}
			else
			{
				from.SendLocalizedMessage( 1010433 ); /* House placement cancellation could result in a
													   * 60 second delay in the return of your deed.
													   */

				from.Target = new HousePlacementTarget( this );
			}
		}

		public abstract BaseHouse GetHouse( Mobile owner );
		public abstract Rectangle2D[] Area{ get; }
		public abstract int LabelNumber{ get; }

		public override void OnSingleClick( Mobile from )
		{
			LabelTo( from, LabelNumber );
		}

		public void OnPlacement( Mobile from, Point3D p )
		{
			if ( Deleted )
			{
				return;
			}
			else if ( !IsChildOf( from.Backpack ) )
			{
				from.SendLocalizedMessage( 1042001 ); // That must be in your pack for you to use it.
			}
			else
			{
				Point3D Center =  new Point3D( p.X - m_Offset.X, p.Y - m_Offset.Y, p.Z - m_Offset.Z );
				Point3D pt = new Point3D( Center );
				ArrayList Move = new ArrayList();
				int i;

				for (i=0;i<this.Area.Length;i++)
				{
					Rectangle2D r = new Rectangle2D( this.Area[i].Start.X + Center.X, this.Area[i].Start.Y+Center.Y, this.Area[i].Width, this.Area[i].Height+2 );//+2 space in front of the house
					for (int x=r.Start.X;x<=r.End.X;x+=8)
					{
						for (int y=r.Start.Y;y<=r.End.Y;y+=8)
						{
							pt.X = x;
							pt.Y = y;
							IPooledEnumerable eable = from.Map.GetObjectsInRange( pt, 8 );

							foreach ( object o in eable )
							{
								if ( o is Item )
								{
									Item item = (Item)o;

									if ( !item.Visible || !r.Contains( item.Location ) )
										continue;

									if ( !item.Movable )
									{
										from.SendLocalizedMessage( 1043287 );//The house could not be created here.  Either something is blocking the house, or the house wolrd not be on valid terrain.
										eable.Free();
										return;
									} 
									else
									{
										Move.Add( item );
									}
								}
								else if ( o is Mobile )
								{
									Mobile m = (Mobile)o;

									if ( !r.Contains( m.Location ) )
										continue;

									if ( m.Player )
									{
										from.SendLocalizedMessage( 1043286 );//The house could not be created here.  A living creature is blocking it
										eable.Free();
										return;
									}
									else
									{
										Move.Add( m );
									}
								}
							}

							eable.Free();

							Tile[][][] statics = from.Map.Tiles.GetStaticBlock( x >> 3, y >> 3 );
							Tile[] land = from.Map.Tiles.GetLandBlock( x >> 3, y >> 3 );

							for (int tx=0;tx<8;tx++)
							{
								for (int ty=0;ty<8;ty++)
								{
									pt.X = (x>>3)*8+tx;
									pt.Y = (y>>3)*8+ty;
									if ( !r.Contains( pt ) )
										continue;

									if ( Math.Abs( Center.Z - land[(ty << 3) + tx].Z ) > 2 )
									{
										from.SendLocalizedMessage( 1043287 );//The house could not be created here.  Either something is blocking the house, or the house wolrd not be on valid terrain.
										return;
									}

									for ( int j = 0; j < statics[tx][ty].Length; j++ )
									{
										Tile t = statics[tx][ty][j];
								
										//Console.WriteLine( "Checking {0} - @({1},{2},{3}+{4})", TileData.ItemTable[t.ID&0x3FFF].Name, x, y, t.Z, TileData.ItemTable[t.ID&0x3FFF].CalcHeight );
										if ( t.Z <= (Center.Z+4) && t.Z+TileData.ItemTable[t.ID&0x3FFF].CalcHeight >= (Center.Z+4) && (TileData.ItemTable[t.ID & 0x3FFF].Flags&TileFlag.Impassable)!=0 )
										{
											//Console.WriteLine( "Blocked by {0} - @({1},{2},{3}+{4})", TileData.ItemTable[t.ID&0x3FFF].Name, x, y, t.Z, TileData.ItemTable[t.ID&0x3FFF].CalcHeight );
											from.SendLocalizedMessage( 1043287 );//The house could not be created here.  Either something is blocking the house, or the house wolrd not be on valid terrain.
											return;
										}
									}
								}//for ty
							}//for tx
						}//for y
					}//for x
				}//for i

				BaseHouse house = GetHouse( from );
				house.MoveToWorld( Center, from.Map );
				this.Delete();

				for (i=0;i<Move.Count;i++)
				{
					if ( Move[i] is Mobile )
						((Mobile)Move[i]).Location = house.BanLocation;
					else if ( Move[i] is Item )
						((Item)Move[i]).Location = house.BanLocation;
				}
			}
		}
	}

	public class StonePlasterHouseDeed : HouseDeed
	{
		[Constructable]
		public StonePlasterHouseDeed() : base( 0x64, new Point3D( 0, 4, 0 ) )
		{
		}

		public StonePlasterHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallOldHouse( owner, 0x64 );
		}

		public override int LabelNumber{ get{ return 1041211; } }
		public override Rectangle2D[] Area{ get{ return SmallOldHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class FieldStoneHouseDeed : HouseDeed
	{
		[Constructable]
		public FieldStoneHouseDeed() : base( 0x66, new Point3D( 0, 4, 0 ) )
		{
		}

		public FieldStoneHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallOldHouse( owner, 0x66 );
		}

		public override int LabelNumber{ get{ return 1041212; } }
		public override Rectangle2D[] Area{ get{ return SmallOldHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class SmallBrickHouseDeed : HouseDeed
	{
		[Constructable]
		public SmallBrickHouseDeed() : base( 0x68, new Point3D( 0, 4, 0 ) )
		{
		}

		public SmallBrickHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallOldHouse( owner, 0x68 );
		}

		public override int LabelNumber{ get{ return 1041213; } }
		public override Rectangle2D[] Area{ get{ return SmallOldHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class WoodHouseDeed : HouseDeed
	{
		[Constructable]
		public WoodHouseDeed() : base( 0x6A, new Point3D( 0, 4, 0 ) )
		{
		}

		public WoodHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallOldHouse( owner, 0x6A );
		}

		public override int LabelNumber{ get{ return 1041214; } }
		public override Rectangle2D[] Area{ get{ return SmallOldHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class WoodPlasterHouseDeed : HouseDeed
	{
		[Constructable]
		public WoodPlasterHouseDeed() : base( 0x6C, new Point3D( 0, 4, 0 ) )
		{
		}

		public WoodPlasterHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallOldHouse( owner, 0x6C );
		}

		public override int LabelNumber{ get{ return 1041215; } }
		public override Rectangle2D[] Area{ get{ return SmallOldHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class ThatchedRoofCottageDeed : HouseDeed
	{
		[Constructable]
		public ThatchedRoofCottageDeed() : base( 0x6E, new Point3D( 0, 4, 0 ) )
		{
		}

		public ThatchedRoofCottageDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallOldHouse( owner, 0x6E );
		}

		public override int LabelNumber{ get{ return 1041216; } }
		public override Rectangle2D[] Area{ get{ return SmallOldHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class BrickHouseDeed : HouseDeed
	{
		[Constructable]
		public BrickHouseDeed() : base( 0x74, new Point3D( -1, 7, 0 ) )
		{
		}

		public BrickHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new GuildHouse( owner );
		}

		public override int LabelNumber{ get{ return 1041219; } }
		public override Rectangle2D[] Area{ get{ return GuildHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class TwoStoryWoodPlasterHouseDeed : HouseDeed
	{
		[Constructable]
		public TwoStoryWoodPlasterHouseDeed() : base( 0x76, new Point3D( -3, 7, 0 ) )
		{
		}

		public TwoStoryWoodPlasterHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new TwoStoryHouse( owner, 0x76 );
		}

		public override int LabelNumber{ get{ return 1041220; } }
		public override Rectangle2D[] Area{ get{ return TwoStoryHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class TwoStoryStonePlasterHouseDeed : HouseDeed
	{
		[Constructable]
		public TwoStoryStonePlasterHouseDeed() : base( 0x78, new Point3D( -3, 7, 0 ) )
		{
		}

		public TwoStoryStonePlasterHouseDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new TwoStoryHouse( owner, 0x78 );
		}

		public override int LabelNumber{ get{ return 1041221; } }
		public override Rectangle2D[] Area{ get{ return TwoStoryHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class TowerDeed : HouseDeed
	{
		[Constructable]
		public TowerDeed() : base( 0x7A, new Point3D( 0, 7, 0 ) )
		{
		}

		public TowerDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new Tower( owner );
		}

		public override int LabelNumber{ get{ return 1041222; } }
		public override Rectangle2D[] Area{ get{ return Tower.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class KeepDeed : HouseDeed
	{
		[Constructable]
		public KeepDeed() : base( 0x7C, new Point3D( 0, 11, 0 ) )
		{
		}

		public KeepDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new Keep( owner );
		}

		public override int LabelNumber{ get{ return 1041223; } }
		public override Rectangle2D[] Area{ get{ return Keep.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class CastleDeed : HouseDeed
	{
		[Constructable]
		public CastleDeed() : base( 0x7E, new Point3D( 0, 16, 0 ) )
		{
		}

		public CastleDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new Castle( owner );
		}

		public override int LabelNumber{ get{ return 1041224; } }
		public override Rectangle2D[] Area{ get{ return Castle.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class LargePatioDeed : HouseDeed
	{
		[Constructable]
		public LargePatioDeed() : base( 0x8C, new Point3D( -4, 7, 0 ) )
		{
		}

		public LargePatioDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new LargePatioHouse( owner );
		}

		public override int LabelNumber{ get{ return 1041231; } }
		public override Rectangle2D[] Area{ get{ return LargePatioHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class LargeMarbleDeed : HouseDeed
	{
		[Constructable]
		public LargeMarbleDeed() : base( 0x96, new Point3D( -4, 7, 0 ) )
		{
		}

		public LargeMarbleDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new LargeMarbleHouse( owner );
		}

		public override int LabelNumber{ get{ return 1041236; } }
		public override Rectangle2D[] Area{ get{ return LargeMarbleHouse.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class SmallTowerDeed : HouseDeed
	{
		[Constructable]
		public SmallTowerDeed() : base( 0x98, new Point3D( 3, 4, 0 ) )
		{
		}

		public SmallTowerDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallTower( owner );
		}

		public override int LabelNumber{ get{ return 1041237; } }
		public override Rectangle2D[] Area{ get{ return SmallTower.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class LogCabinDeed : HouseDeed
	{
		[Constructable]
		public LogCabinDeed() : base( 0x9A, new Point3D( 1, 6, 0 ) )
		{
		}

		public LogCabinDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new LogCabin( owner );
		}

		public override int LabelNumber{ get{ return 1041238; } }
		public override Rectangle2D[] Area{ get{ return LogCabin.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class SandstonePatioDeed : HouseDeed
	{
		[Constructable]
		public SandstonePatioDeed() : base( 0x9C, new Point3D( -1, 4, 0 ) )
		{
		}

		public SandstonePatioDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SandStonePatio( owner );
		}

		public override int LabelNumber{ get{ return 1041239; } }
		public override Rectangle2D[] Area{ get{ return SandStonePatio.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class VillaDeed : HouseDeed
	{
		[Constructable]
		public VillaDeed() : base( 0x9E, new Point3D( 3, 6, 0 ) )
		{
		}

		public VillaDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new TwoStoryVilla( owner );
		}

		public override int LabelNumber{ get{ return 1041240; } }
		public override Rectangle2D[] Area{ get{ return TwoStoryVilla.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class StoneWorkshopDeed : HouseDeed
	{
		[Constructable]
		public StoneWorkshopDeed() : base( 0xA0, new Point3D( -1, 4, 0 ) )
		{
		}

		public StoneWorkshopDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallShop( owner, 0xA0 );
		}

		public override int LabelNumber{ get{ return 1041241; } }
		public override Rectangle2D[] Area{ get{ return SmallShop.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class MarbleWorkshopDeed : HouseDeed
	{
		[Constructable]
		public MarbleWorkshopDeed() : base( 0xA2, new Point3D( -1, 4, 0 ) )
		{
		}

		public MarbleWorkshopDeed( Serial serial ) : base( serial )
		{
		}

		public override BaseHouse GetHouse( Mobile owner )
		{
			return new SmallShop( owner, 0xA2 );
		}

		public override int LabelNumber{ get{ return 1041242; } }
		public override Rectangle2D[] Area{ get{ return SmallShop.Area; } }

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}