using System;
using Server.Targeting;
using Server.Network;
using Server.Misc;

namespace Server.Spells.Third
{
	public class WallOfStoneSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Wall of Stone", "In Sanct Ylem",
				SpellCircle.Third,
				Reagent.MandrakeRoot,
				Reagent.Nightshade
			);

		public WallOfStoneSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( IPoint3D p )
		{
			if ( !Caster.CanSee( p ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckSequence() )
			{
				if ( p is Item )
					p = ((Item)p).GetWorldLocation();

				int dx = Caster.Location.X - p.X;
				int dy = Caster.Location.Y - p.Y;
				int rx = (dx - dy) * 44;
				int ry = (dx + dy) * 44;

				bool eastToWest;

				if ( rx >= 0 && ry >= 0 )
				{
					eastToWest = false;
				}
				else if ( rx >= 0 )
				{
					eastToWest = true;
				}
				else if ( ry >= 0 )
				{
					eastToWest = true;
				}
				else
				{
					eastToWest = false;
				}

				if ( Caster != p )
					Caster.Direction = Caster.GetDirectionTo( p );

				Effects.PlaySound( p, Caster.Map, 0x1F6 );

				for ( int i = -1; i <= 1; ++i )
					new InternalItem( p, eastToWest, i, Caster.Map );
			}

			FinishSequence();
		}

		[DispellableField]
		private class InternalItem : Item
		{
			private Timer m_Timer;

			public InternalItem( IPoint3D p, bool eastToWest, int offset, Map map ) : base( 0x80 )
			{
				MoveToWorld( new Point3D( eastToWest ? p.X + offset : p.X, eastToWest ? p.Y : p.Y + offset, p.Z ), map );
				Movable = false;

				Effects.SendTargetEffect( this, 0x376A, 16 );

				m_Timer = new InternalTimer( this );
				m_Timer.Start();
			}

			public InternalItem( Serial serial ) : base( serial )
			{
				m_Timer = new InternalTimer( this );
				m_Timer.Start();
			}

			public override void Serialize( GenericWriter writer )
			{
				base.Serialize( writer );

				writer.Write( (int) 0 ); // version
			}

			public override void Deserialize( GenericReader reader )
			{
				base.Deserialize( reader );

				int version = reader.ReadInt();
			}

			public override void OnAfterDelete()
			{
				m_Timer.Stop();

				base.OnAfterDelete();
			}

			private class InternalTimer : Timer
			{
				private InternalItem m_Item;

				public InternalTimer( InternalItem item ) : base( TimeSpan.FromSeconds( 10.0 ) )
				{
					m_Item = item;
				}

				protected override void OnTick()
				{
					m_Item.Delete();
				}
			}
		}

		private class InternalTarget : Target
		{
			private WallOfStoneSpell m_Owner;

			public InternalTarget( WallOfStoneSpell owner ) : base( 12, true, TargetFlags.None )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is IPoint3D )
				{
					m_Owner.Target( (IPoint3D)o );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}