using System;
using Server;
using Server.Targeting;
using Server.Items;

namespace Server.Targets
{
	public class BladedItemTarget : Target
	{
		private Item m_Item;

		private static int[] m_TreeIDs = new int[]
				{
					0x0CD0, 0x0CD3, 0x0CD6,
					0x0CD8, 0x0CDA, 0x0CDD,
					0x0CE0, 0x0CE3, 0x0CE6,
					0x0CCA, 0x0CCB, 0x0CCC,
					0x0CCD, 0x0C12, 0x0CB8,
					0x0CB9, 0x0CBA, 0x0CBB
				};

		public BladedItemTarget( Item item ) : base( 2, false, TargetFlags.None )
		{
			m_Item = item;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( m_Item.Deleted )
				return;

			if ( targeted is StaticTarget )
			{
				StaticTarget obj = (StaticTarget)targeted;

				bool isTree = false;

				for ( int i = 0; i < m_TreeIDs.Length && !isTree; ++i )
				{
					if ( obj.ItemID == m_TreeIDs[i] )
						isTree = true;
				}

				if ( isTree )
				{
					Item item = new Kindling();

					if ( from.PlaceInBackpack( item ) )
					{
						from.SendLocalizedMessage( 500491 ); // You put some kindling into your backpack.
						from.SendLocalizedMessage( 500492 ); // An axe would probably get you more wood.
					}
					else
					{
						from.SendLocalizedMessage( 500490 ); // You can't place any kindling into your backpack!

						item.Delete();
					}
				}
				else
				{
					from.SendLocalizedMessage( 500494 ); // You can't use a bladed item on that!
				}
			}
			else if ( targeted is ICarvable )
			{
				((ICarvable)targeted).Carve( from, m_Item );
			}
			else
			{
				from.SendLocalizedMessage( 500494 ); // You can't use a bladed item on that!
			}
		}
	}
}